#!/usr/bin/btpython
# -*- coding:utf-8 -*-


import time
import hashlib
import sys
import os
import json

sys.path.insert(0, '/www/server/panel/class/')
import public  # NOQA


class BTAPI:
    __BT_KEY = 'APIKEY'
    __BT_PANEL = 'http://127.0.0.1:8888'

    # 如果希望多台面板，可以在实例化对象时，将面板地址与密钥传入
    def __init__(self, bt_panel=None, bt_key=None):
        if bt_panel:
            self.__BT_PANEL = bt_panel
            self.__BT_KEY = bt_key

    def getpid(self, domain):
        SQL = public.M('sites')
        jg = SQL.where("name=?", (domain,)).field('id').find()
        if 'id' in jg:
            return jg['id']
        else:
            raise Exception("not found site: "+domain)

    # 申请letsencrypt ssl证书
    def pssl(self, domain):
        url = self.__BT_PANEL+'/acme?action=apply_cert_api'
        pid = self.getpid(domain)

        pd = self.__get_key_data()
        pd['domains'] = '["www.{0}","{0}"]'.format(domain)
        pd['auth_type'] = 'http'
        pd['auth_to'] = pid
        pd['auto_wildcard'] = 0
        pd['id'] = pid

        result = json.loads(self.__http_post_cookie(url, pd))
        if result['status']:
            return self.setssl(domain)
        return result

    def setssl(self, domain):
        url = self.__BT_PANEL+'/site?action=SetSSL'
        kf = "/www/server/panel/vhost/letsencrypt/www.{}/privkey.pem".format(
            domain)
        cf = "/www/server/panel/vhost/letsencrypt/www.{}/fullchain.pem".format(
            domain)
        with open(kf) as f:
            kfs = f.read()
        with open(cf) as f:
            cfs = f.read()

        pd = self.__get_key_data()
        pd.update(type=1, siteName=domain, key=kfs, csr=cfs)
        result = self.__http_post_cookie(url, pd)
        return json.loads(result)

    # 根据 ~/scripts/dossl.txt 申请letsencrypt ssl证书
    # 失败日志 ~/scripts/ssllog/domain
    def dofile_pssl(self, dofile=None):
        spath = "{}/scripts/".format(os.environ['HOME'])
        if dofile is None:
            dofile = spath+"dossl.txt"
        ssllog = spath+"ssllog"
        if not os.path.exists(ssllog):
            os.makedirs(ssllog)

        with open(dofile) as f:
            dolist = f.readlines()

        for i in dolist:
            domain = i.replace('\n', '')
            jg = self.pssl(domain)
            if not jg['status']:
                with open(os.path.join(ssllog, '{}'.format(domain)), 'a') as f:
                    f.write(json.dumps(jg['msg'], ensure_ascii=False))

    # 计算MD5
    def __get_md5(self, s):
        m = hashlib.md5()
        m.update(s.encode('utf-8'))
        return m.hexdigest()

    # 构造带有签名的关联数组
    def __get_key_data(self):
        now_time = int(time.time())
        pd = {
            'request_token': self.__get_md5(str(now_time) + '' + self.__get_md5(self.__BT_KEY)),
            'request_time': now_time
        }
        return pd

    # 发送POST请求并保存Cookie
    # @url 被请求的URL地址(必需)
    # @data POST参数，可以是字符串或字典(必需)
    # @timeout 超时时间默认1800秒
    # return string
    def __http_post_cookie(self, url, pd, timeout=1800):
        cookie_file = './' + self.__get_md5(self.__BT_PANEL) + '.cookie'
        with open(cookie_file, "a") as file:
            file.write("# Netscape HTTP Cookie File\n")
        if sys.version_info[0] == 2:
            # Python2
            import urllib
            import urllib2
            import ssl
            import cookielib

            # 创建cookie对象
            cookie_obj = cookielib.MozillaCookieJar(cookie_file)

            # 加载已保存的cookie
            if os.path.exists(cookie_file):
                cookie_obj.load(cookie_file, ignore_discard=True,
                                ignore_expires=True)

            ssl._create_default_https_context = ssl._create_unverified_context

            data = urllib.urlencode(pd)
            req = urllib2.Request(url, data)
            opener = urllib2.build_opener(
                urllib2.HTTPCookieProcessor(cookie_obj))
            response = opener.open(req, timeout=timeout)

            # 保存cookie
            cookie_obj.save(ignore_discard=True, ignore_expires=True)
            return response.read()
        else:
            # Python3
            import urllib.request
            import ssl
            import http.cookiejar
            cookie_obj = http.cookiejar.MozillaCookieJar(cookie_file)
            cookie_obj.load(cookie_file, ignore_discard=True,
                            ignore_expires=True)
            handler = urllib.request.HTTPCookieProcessor(cookie_obj)
            data = urllib.parse.urlencode(pd).encode('utf-8')
            req = urllib.request.Request(url, data)
            opener = urllib.request.build_opener(handler)
            response = opener.open(req, timeout=timeout)
            cookie_obj.save(ignore_discard=True, ignore_expires=True)
            result = response.read()
            if type(result) == bytes:
                result = result.decode('utf-8')
            return result
